#!/usr/local/bin/perl -w

###########################################################################
#
# sval2dups.pl version 0.2
#
###########################################################################
#
# This program checks for duplicate instance IDs and contexts in a 
# SOURCE file which has data in Senseval-2 data format (test or training 
# file). The line number of these duplicates along with the complete 
# instance information is printed to standard output. 
#
# USAGE :  sval2dups.pl OPTIONS SOURCE
#
#	--version       Prints the version number.
#	--help		Prints help message.
#
# Following messages are printed on standard output.
# Number of instances in SOURCE.
# Information about pairs of instances which have the same context.
# Each such pair is classified into four categories listed below, based 
# on their instance IDs and sense IDs.
#
#	Same Instance IDs, Same Sense IDs 	: TRUE DUPLICATES
#	Same Instance IDs, Different Sense IDs	: OK (multiple senses) 
#	Different Instance IDs, Same Sense IDs  : DUPLICATE ENTRIES 
#						  (differing Instance IDs)
#	Different Instance IDs, Different Sense IDs : ERROR 
#
# Summarizing information on the number of such pairs in each category is
# also provided.
# Information of pairs of instances which have the same instance ID but 
# differing contexts is also provided. Such pairs, if existing, would 
# also qualify as ERRORS.
#
###########################################################################
#
# Copyright (C) 2000-2002
#
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
###########################################################################

# START OF PROGRAM
use Getopt::Long;

# Check if any options specified
GetOptions("version", "help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }

# if version has been requested, show version
if ( defined $opt_version ) {
    $opt_version = 1;
    &showVersion();
    exit; }

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 0 ) {
    &usage();
    exit;}

$inpfl = shift();
# Open file
open(INP,"<$inpfl");

$instNo = 0;
$senseid = "NO SENSE ID";
# Read lines from input file
while($ln = <INP>) {
    $lnno++;

    ### CAPTURE INSTANCE ID INFORMATION
    if($ln =~ /^\s*<instance id=\"([^\"]+)\"/) {
	$id = $1;
	# Capture instance ID
	$instH{$id}++;
	$instNo++;
    }
    
    ### CAPTURE ANSWER ID INFORMATION
    if($ln =~ /^\s*<answer instance=\"[^\"]+\"\s+senseid=\"([^\"]+)\"/) {
        $senseid = $1; }
    
    if($ln =~ /^\s*<context>\s*$/) {
	# capture context
		$tmplnno = 0;
	$con = "";
	while($cline = <INP>) {
	    $tmplnno++;
	    if($cline =~ /^\s*<\/context>\s*$/)  { last; }
	    $con = $con.$cline;	
	}
	$conctr{$con}++;
	
	# store context for dup IDs
	$dupHcon{$id}{$instH{$id}} = $con;
	$lineNum{$id}{$instH{$id}} = $lnno;
	
	if(exists $conH{$con})	{ 
	    if($conctr{$con} == 2) {
		# Enter details of earlier found instance
		$dupconH{$con}{1}{LineNo} = $conH{$con}{LineNo};
		$dupconH{$con}{1}{InstID} = $conH{$con}{InstID};
		$dupconH{$con}{1}{SenseID} = $conH{$con}{SenseID};
	    }
	    $dupconH{$con}{$conctr{$con}}{LineNo} = $lnno;
	    $dupconH{$con}{$conctr{$con}}{InstID} = $id;
	    $dupconH{$con}{$conctr{$con}}{SenseID} = $senseid;
	}
	else { 
	    $conH{$con}{LineNo} = $lnno; 
	    $conH{$con}{InstID} = $id;
	    $conH{$con}{SenseID} = $senseid;
	}
	$lnno += $tmplnno;
    }
}
print "Total number of instances in SOURCE is $instNo\n\n";
close(INP);

#### DUPLICATE CONTEXTS
$okcnt = 0; $dupcnt = 0; $trudupcnt = 0; $errcnt = 0;
print "XXXXXXXXXXXXXXXXXX\n";
print "DUPLICATE CONTEXTS\n";
print "XXXXXXXXXXXXXXXXXX\n\n";

## BEGIN CHECK FOR EACH OF THE CONTEXTS WHICH HAS DUPLICATES
foreach $unicontext(keys %dupconH)
{
    # Examine each pair of instances
    foreach $ConNo1(keys %{$dupconH{$unicontext}})
    {
	# print "ONE: $ConNo1\n";
	foreach $ConNo2(keys %{$dupconH{$unicontext}})
	{
	    # print "TWO: $ConNo2\n";
	    # Dont want to compare instance with itself ! and
	    # Dont want to consider comparisons already made
	    if($ConNo2 <= $ConNo1) 	{ next; }
	    
	    # Compare the two instances
	    if($dupconH{$unicontext}{$ConNo1}{InstID} eq $dupconH{$unicontext}{$ConNo2}{InstID} && $dupconH{$unicontext}{$ConNo1}{SenseID} eq $dupconH{$unicontext}{$ConNo2}{SenseID}) {
		# TRUE DUPLICATES
		$trudupcnt++;
		print "TRUE DUPLICATES\n\n";
		# Printing Details of the pair
		print "INSTANCE1 (Line Number $dupconH{$unicontext}{$ConNo1}{LineNo})\nInstance ID\t: $dupconH{$unicontext}{$ConNo1}{InstID}\n";
		print "SenseID\t\t: $dupconH{$unicontext}{$ConNo1}{SenseID}\n";
		print "INSTANCE2 (Line Number $dupconH{$unicontext}{$ConNo2}{LineNo})\nInstance ID\t: $dupconH{$unicontext}{$ConNo2}{InstID}\n";
		print "SenseID\t\t: $dupconH{$unicontext}{$ConNo2}{SenseID}\n";
		print "CONTEXT\t: $unicontext\n\n";
		next;
	    }
	    if($dupconH{$unicontext}{$ConNo1}{InstID} ne $dupconH{$unicontext}{$ConNo2}{InstID} && $dupconH{$unicontext}{$ConNo1}{SenseID} ne $dupconH{$unicontext}{$ConNo2}{SenseID}) {
		# ERROR 
		$errcnt++;
		print "ERROR\n\n";
		# Printing Details of the pair
		print "INSTANCE1 (Line Number $dupconH{$unicontext}{$ConNo1}{LineNo})\nInstance ID\t: $dupconH{$unicontext}{$ConNo1}{InstID}\n";
		print "SenseID\t\t: $dupconH{$unicontext}{$ConNo1}{SenseID}\n";
		print "INSTANCE2 (Line Number $dupconH{$unicontext}{$ConNo2}{LineNo})\nInstance ID\t: $dupconH{$unicontext}{$ConNo2}{InstID}\n";
		print "SenseID\t\t: $dupconH{$unicontext}{$ConNo2}{SenseID}\n";
		print "CONTEXT\t: $unicontext\n\n";
		next;
	    }
	    if($dupconH{$unicontext}{$ConNo1}{InstID} ne $dupconH{$unicontext}{$ConNo2}{InstID} && $dupconH{$unicontext}{$ConNo1}{SenseID} eq $dupconH{$unicontext}{$ConNo2}{SenseID}) {
		# DUPLICATES 
		$dupcnt++;
		print "DUPLICATES (Differing Instance IDs)\n\n";
		# Printing Details of the pair
		print "INSTANCE1 (Line Number $dupconH{$unicontext}{$ConNo1}{LineNo})\nInstance ID\t: $dupconH{$unicontext}{$ConNo1}{InstID}\n";
		print "SenseID\t\t: $dupconH{$unicontext}{$ConNo1}{SenseID}\n";
		print "INSTANCE2 (Line Number $dupconH{$unicontext}{$ConNo2}{LineNo})\nInstance ID\t: $dupconH{$unicontext}{$ConNo2}{InstID}\n";
		print "SenseID\t\t: $dupconH{$unicontext}{$ConNo2}{SenseID}\n";
		print "CONTEXT\t: $unicontext\n\n";
		next;
	    }
	    # OK  
	    $okcnt++;
	    print "OK (multiple senses)\n\n";
	    # Printing Details of the pair
	    print "INSTANCE1 (Line Number $dupconH{$unicontext}{$ConNo1}{LineNo})\nInstance ID\t: $dupconH{$unicontext}{$ConNo1}{InstID}\n";
	    print "SenseID\t\t: $dupconH{$unicontext}{$ConNo1}{SenseID}\n";
	    print "INSTANCE2 (Line Number $dupconH{$unicontext}{$ConNo2}{LineNo})\nInstance ID\t: $dupconH{$unicontext}{$ConNo2}{InstID}\n";
	    print "SenseID\t\t: $dupconH{$unicontext}{$ConNo2}{SenseID}\n";
	    print "CONTEXT\t: $unicontext\n\n";
	}	
    }
}

print "Total number of pairs of instances which have the same contexts and are:\n\n";
print "TRUE DUPLICATES (same instance IDs and same sense IDs)\t: $trudupcnt\nDUPLICATES (differing instance IDs and same sense IDs)\t: $dupcnt\n\n";
print "ERRORS (Differing instance IDs and differing sense IDs)\t: $errcnt\nOK (Same instance IDs and differing sense IDs)\t: $okcnt\n\n";

### DUPLICATE INSTANCE IDS
print "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n";
print "DUPLICATE INSTANCE IDs AND DIFFERING CONTEXTS\n";
print "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n\n";

foreach $tmpID(keys %dupHcon) {
    foreach $tmpNo1(keys %{$dupHcon{$tmpID}}) {
	foreach $tmpNo2(keys %{$dupHcon{$tmpID}}) {        
	    if($tmpNo1 <= $tmpNo2) 	{ next; }
	    # Check for instances with same instance ID and differing contexts
	    if($dupHcon{$tmpID}{$tmpNo1} ne $dupHcon{$tmpID}{$tmpNo2}) {
		print "ERROR (Same instance IDs but differing contexts)\n\n";
		# Printing Details of the pair
		print "Instance ID\t: $tmpID\n\n";
		print "INSTANCE1 (Line Number $lineNum{$tmpID}{$tmpNo1})\nContext \t: $dupHcon{$tmpID}{$tmpNo1}\n";
		print "INSTANCE2 (Line Number $lineNum{$tmpID}{$tmpNo2})\nContext \t: $dupHcon{$tmpID}{$tmpNo2}\n";
	    }
	}	
    }
}

# Subroutine to print help message
sub help{
    print "Usage: sval2dups.pl OPTIONS DESTINATION SOURCE\n\n"; 
    print "This program checks for duplicate instance IDs and contexts in a SOURCE file which has data in Senseval-2 data format. The line number of these duplicates along with the complete instances is printed to standard output.\n\n";
    print "OPTIONS:\n\n";
    print "  --version    Prints the version number.\n";
    print "  --help       Prints the help message.\n\n";}

sub usage{
    print "Usage: sval2dups.pl OPTIONS SOURCE\n"; 
    AskHelp();}
# Message describing way to get help
sub AskHelp{
    print "Type sval2dups.pl --help for help.\n";}

# function to output the version number
sub showVersion
{
    print STDERR "sval2dups.pl      -        version 0.2\n";
    print STDERR "A component of the Sval2Check 0.1\n";
    print STDERR "Copyright (C) 2000-2003, Saif Mohammad and Ted Pedersen\n";
}


